﻿using ProjectRepairWork.Repositories;
using ProjectRepairWork.Reports;
using Unity;

namespace ProjectRepairWork.Forms
{
    public partial class FormRepairReport : Form
    {
        private readonly IUnityContainer _container;


        public FormRepairReport(IUnityContainer container, IContractorsRepository contractorRepository)
        {
            _container = container ?? throw new ArgumentNullException(nameof(container));
            InitializeComponent();

            comboBoxContractor.DataSource = contractorRepository.ReadContractors();
            comboBoxContractor.DisplayMember = "CompanyName";
            comboBoxContractor.ValueMember = "Id";
        }

        private void buttonMakeReport_Click(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(textBoxFilePath.Text))
                {
                    throw new Exception("Отсутствует имя файла для отчета");
                }

                if (comboBoxContractor.SelectedIndex < 0)
                {
                    throw new Exception("Не выбран подрядчик");
                }

                if (dateTimePickerDateEnd.Value <= dateTimePickerDateBegin.Value)
                {
                    throw new Exception("Дата начала должна быть раньше даты окончания");
                }

                if (_container.Resolve<TableReport>().CreateTable(textBoxFilePath.Text, (int)comboBoxContractor.SelectedValue!,
            dateTimePickerDateBegin.Value, dateTimePickerDateEnd.Value))
                {
                    MessageBox.Show("Документ сформирован", "Формирование документа", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    MessageBox.Show("Возникли ошибки при формировании документа.Подробности в логах", "Формирование документа", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при создании очета", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonSelectFilePath_Click(object sender, EventArgs e)
        {
            var sfd = new SaveFileDialog()
            {
                Filter = "Excel Files | *.xlsx"
            };

            if (sfd.ShowDialog() != DialogResult.OK)
            {
                return;
            }

            textBoxFilePath.Text = sfd.FileName;
        }
    }
}
