﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectRepairWork.Entities;
using ProjectRepairWork.Entities.Enums;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectRepairWork.Repositories.Implementation;

public class BuildingMaterialsRepository : IBuildingMaterialsRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<BuildingMaterialsRepository> _logger;
    public BuildingMaterialsRepository(IConnectionString connectionString, ILogger<BuildingMaterialsRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CreateBuildingMaterials(BuildingMaterials buildingMaterials)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(buildingMaterials));

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryInsert = @"
INSERT INTO BuildingMaterials (MaterialsType)
VALUES (@MaterialsType)";
            connection.Execute(queryInsert, buildingMaterials);
        }

        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }
    public void UpdateBuildingMaterials(BuildingMaterials buildingMaterials)
    {
        _logger.LogInformation("Редактирование объекта");
        _logger.LogDebug("Объект: {json}",
            JsonConvert.SerializeObject(buildingMaterials));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryUpdate = @"
                UPDATE BuildingMaterials
                SET
                BuildingMaterials=@BuildingMaterials
                WHERE Id=@Id";
            connection.Execute(queryUpdate, buildingMaterials);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при редактировании объекта");
            throw;
        }
    }

    public void DeletedBuildingMaterials(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryDelete = @"
                DELETE FROM BuildingMaterials
                WHERE Id=@id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }

    public IEnumerable<BuildingMaterials> ReadBuildingMaterials()
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = "SELECT * FROM buildingmaterials";
            var buildingMaterials = connection.Query<BuildingMaterials>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}",
            JsonConvert.SerializeObject(buildingMaterials));
            return buildingMaterials;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }

    public BuildingMaterials ReadBuildingMaterialsById(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"
                SELECT * FROM BuildingMaterials
                WHERE Id=@id";
            var buildingMaterials = connection.QueryFirst<BuildingMaterials>(querySelect, new { id });
            _logger.LogDebug("Найденный объект: {json}",
                JsonConvert.SerializeObject(buildingMaterials));
            return buildingMaterials;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }


}
